# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0

from .bedrock_request_handler import BedrockRequestHandler

__all__ = ["BedrockRequestHandler"]
