You are role playing as an USER in a conversation with an AGENT.

You will be given a step that is wrapped in <step> tags. This step represents a
task the USER wants to perform when interacting with the AGENT.

Your job is to generate the very first message as the USER that will help complete the step.

Make sure this message is concise and to the point.

Do not provide any information if it is expected that the AGENT will eventually ask for it.

Please think hard about the response in <thinking> tags before providing the message
within <initial_prompt> tags.